/**
 * Yvonne V. Richardson, BSCS, M. Ed.
 * yvonne.richardson@comcast.net
 * http://yvonnerichardson.weebly.com
 * Android Java Application
 * Resume for Phones and Tablets
 * Application Deployed 2014-09-25
 * Optimal presentation is on a 7" tablet in landscape mode
 * Application displays well in portrait mode also
 */
package com.example.resume;

import java.io.BufferedReader;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import android.app.Fragment;
import android.content.Context;
import android.content.Intent;
import android.content.res.AssetManager;
import android.content.res.Configuration;
import android.content.res.Resources;
import android.graphics.Rect;
import android.graphics.Typeface;
import android.net.Uri;
import android.os.Bundle;
import android.os.Environment;
import android.view.Display;
import android.view.LayoutInflater;
import android.view.View;
import android.view.View.OnClickListener;
import android.view.ViewGroup;
import android.view.ViewGroup.LayoutParams;
import android.view.WindowManager;
import android.widget.RelativeLayout;
import android.widget.TextView;

/**
 * A fragment representing a single Topic detail screen. This fragment is either
 * contained in a {@link TopicListActivity} in two-pane mode (on tablets) or a
 * {@link TopicDetailActivity} on handsets.
 */
public class TopicDetailFragment extends Fragment {
	/**
	 * The fragment argument representing the item ID that this fragment
	 * represents.
	 */
	public static final String ARG_ITEM_ID = "item_id";

	public String mItem;

	public String strFragName;

	/**
	 * Mandatory empty constructor for the fragment manager to instantiate the
	 * fragment (e.g. upon screen orientation changes).
	 */
	public TopicDetailFragment() {
	}

	private int addElement(Context ct, ViewGroup parent,
			RelativeLayout.LayoutParams rlp, String sText, int intUniqueId,
			int eFormat) {
		// Add the fragment objects to the parent view
		// There is one element per field
		// Returns the id of the element that was added to the parent View
		TextView tv = new TextView(ct.getApplicationContext());
		Resources res = ct.getResources();

		// numeric values for case statement constants

		int eFormatNormal = 0;

		// View Formatting
		int intClrLinkId = res.getIdentifier("clrLink", "color", getActivity()
				.getPackageName());
		int intClrNmlTxtId = res.getIdentifier("clrNormalText", "color",
				getActivity().getPackageName());
		int intClrInvtTxtId = res.getIdentifier("clrInvertText", "color",
				getActivity().getPackageName());

		int intClrLink = res.getColor(intClrLinkId);
		int intClrNmlTxt = res.getColor(intClrNmlTxtId);
		int intClrInvtTxt = res.getColor(intClrInvtTxtId);

		int intDimVertId = res.getIdentifier("app_vert_margin", "dimen",
				getActivity().getPackageName());
		int intDimHorzId = res.getIdentifier("app_horz_margin", "dimen",
				getActivity().getPackageName());
		int intDimBtmId = res.getIdentifier("app_vert_margin", "dimen",
				getActivity().getPackageName());
		int intDimThinId = res.getIdentifier("app_thin_margin", "dimen",
				getActivity().getPackageName());

		int intDimVert = res.getDimensionPixelSize(intDimVertId);
		int intDimHorz = res.getDimensionPixelSize(intDimHorzId);
		int intDimBtm = res.getDimensionPixelSize(intDimBtmId);
		int intDimThin = res.getDimensionPixelSize(intDimThinId);

		String strTools = "Tools and Techniques: ";

		// TextView Content
		tv.setText((CharSequence) sText);
		tv.setId(intUniqueId + tv.getId()); // reasonably unique id

		// TextView Format
		rlp.height = LayoutParams.WRAP_CONTENT;
		rlp.width = LayoutParams.WRAP_CONTENT;

		tv.setPadding(intDimHorz, intDimVert, intDimHorz, intDimThin);
		tv.setTextColor(intClrNmlTxt);
		tv.setTypeface(null, Typeface.NORMAL);

		String fmtName = chooseFormat(eFormat);
		fmtName = fmtName.toLowerCase();
		if (fmtName.equalsIgnoreCase("business")) {
			// eFormatBusiness
			tv.setTypeface(null, Typeface.BOLD);
		} else if (fmtName.equalsIgnoreCase("title")) {
			// eFormatTitle
			tv.setTextColor(intClrNmlTxt);
			tv.setTypeface(null, Typeface.ITALIC);
		} else if (fmtName.equalsIgnoreCase("url")) {
			tv.setTextColor(intClrLink);
			tv.setClickable(true);
			tv.setOnClickListener(new OnClickListener() {
				@Override
				public void onClick(View v) {
					openBrowser(v);
				}
			});
		} else if (fmtName.equalsIgnoreCase("toolsandtechniques")) {
			// eFormatToolsAndTechs
			String strContent = strTools + sText;
			tv.setText((CharSequence) strContent);
		} else {
			tv.setPadding(intDimHorz, intDimVert, intDimHorz, intDimThin);
			tv.setTextColor(intClrNmlTxt);
			tv.setTypeface(null, Typeface.NORMAL);
		} // fmtName

		tv.setLayoutParams(rlp);
		parent.addView(tv); // add the inflated layout to the hierarchy
		return tv.getId();
	}

	private int chooseFormat(String strFieldName) {
		// determine TextView formatting based on name of CSV column
		// Special formatting is based on the name of the CSV element
		int eFormatNormal = 0;
		int eFormatBusiness = 1;
		int eFormatTitle = 2;
		int eFormatURL = 3;
		int eFormatExperience = 4;
		int eFormatToolsAndTechs = 5;

		int eFormat = 0;

		Resources res = getResources();

		// load names of topic fragments into array
		String[] strCSVNames = res.getStringArray(R.array.aryCSVNames);

		for (int i = 0; i < strCSVNames.length; i++) {
			if (strFieldName.equalsIgnoreCase(strCSVNames[i])) {
				eFormat = i + 1;
				i = strCSVNames.length + 1; // stop loop
			}
		}

		return eFormat;
	} // chooseFormat

	private String chooseFormat(int eFormat) {
		// determine the name of the TextView formatting
		// based on the number of the CSV column stored
		// in the resource array

		Resources res = getResources();
		String strCSVName = "";

		// load names of topic fragments into array
		String[] strCSVNames = res.getStringArray(R.array.aryCSVNames);
		if (eFormat != 0) {
			strCSVName = strCSVNames[eFormat - 1];
		}
		return strCSVName;

	} // chooseFormat

	private void formatFragment(Context ct, ViewGroup parent,
			ArrayList<List<String>> TContent) {
		// parent is the detail fragment to which to add content
		// It is a ScrollView that was inflated previous to the
		// call to this method
		// ArrayList<List<String>> contains the entire file to format and
		// display
		List<String> strFieldContents = null;
		RelativeLayout rlColumn = null;
		int prevRow = 0;
		int currRow = 0;
		int prevView = 0;
		int currView = 0;
		int eFormat = 0;

		boolean boolFirstRow = true;
		boolean bLineWasSplit = false;
		String rlTag = "rlTag";

		// the zeroth list in the array contains column names
		List<String> strFieldNames = TContent.get(0);

		// the size of the list is the number of columns

		int intColumns = strFieldNames.size();
		int intUniqueId = 0; // ensures that TextView objects get unique Ids

		// get relative layout in parent
		try {
			rlColumn = (RelativeLayout) parent.findViewWithTag(rlTag);
		} catch (Exception e) {
			e.printStackTrace();
		}

		for (int i = 1; i < TContent.size(); i++) {
			// outer loop processes the array, one line at a time
			// thereby processing the entire input file
			// get the i-th list in the array
			strFieldContents = TContent.get(i);

			TextView tvSpace = new TextView(ct.getApplicationContext());
			tvSpace.setText((CharSequence) " ");
			tvSpace.setId(prevRow + 1); // should be unique

			RelativeLayout.LayoutParams rlSpace = new RelativeLayout.LayoutParams(
					20, 60);

			rlSpace.addRule(RelativeLayout.BELOW, prevRow);
			rlSpace.height = LayoutParams.WRAP_CONTENT;
			rlSpace.width = LayoutParams.WRAP_CONTENT;

			tvSpace.setLayoutParams(rlSpace);
			rlColumn.addView(tvSpace); // add the inflated layout to the
										// hierarchy
			prevRow = tvSpace.getId();
			bLineWasSplit = false;
			for (int j = 0; j < intColumns; j++) {
				// inner loop processes the list of strings,
				// one column at a time. Unless otherwise
				// noted, columns should display from left to right
				//
				// Get the j-th element in the array and convert it
				// to a type that can be stored in a TextView object
				String sText = strFieldContents.get(j); // content

				// Special formatting is based on the name of the CSV element
				String strFieldName = strFieldNames.get(j).toLowerCase();
				eFormat = chooseFormat(strFieldName);

				RelativeLayout.LayoutParams rlp = new RelativeLayout.LayoutParams(
						20, 20);
				if (boolFirstRow == true) { // first entry
					if (j == 0) {
						rlp.addRule(RelativeLayout.ALIGN_PARENT_TOP);
					}
				} else if (i > 1) {
					rlp.addRule(RelativeLayout.BELOW, prevRow);
				}

				// determine screen position of layout element
				// if the screen is in portrait mode, insert
				// a CRLF after every other element
				// load names of CSV columns into array
				Resources res = getResources();
				String[] strCSVNames = res.getStringArray(R.array.aryCSVNames);
				int intOrientation = res.getConfiguration().orientation;

				if ((intOrientation == Configuration.ORIENTATION_PORTRAIT)
						&& (j == 2)) {
					// CSV formatting for narrow screen requires
					// CRLF
					// 2 is about the maximum number of columns that fit on a
					// row
					if (bLineWasSplit) {
						rlp.addRule(RelativeLayout.BELOW, currRow);
						rlp.addRule(RelativeLayout.RIGHT_OF, prevView);
					} else {
						rlp.removeRule(RelativeLayout.ALIGN_PARENT_TOP);
						rlp.addRule(RelativeLayout.BELOW, currRow);
						// carriage return
						rlp.removeRule(RelativeLayout.RIGHT_OF);
						rlp.addRule(RelativeLayout.ALIGN_PARENT_LEFT);
						bLineWasSplit = true;
						prevRow = currRow;
					}
				} else if (j == 0) {
					// the first element in the row is leftmost
					rlp.addRule(RelativeLayout.ALIGN_PARENT_LEFT);
				} else if (eFormat > 0) { // special format
					if ((strCSVNames[eFormat - 1].toLowerCase().contains("url"))
							&& (intOrientation == Configuration.ORIENTATION_PORTRAIT)) {
						// CSV formatting for URL on a narrow screen requires
						// CRLF
						rlp.removeRule(RelativeLayout.ALIGN_PARENT_TOP);
						rlp.addRule(RelativeLayout.BELOW, currRow);
						// carriage return
						rlp.removeRule(RelativeLayout.RIGHT_OF);
						rlp.addRule(RelativeLayout.ALIGN_PARENT_LEFT);
						prevRow = currRow;
					} else if (strCSVNames[eFormat - 1].toLowerCase().contains(
							"tools")
							|| strCSVNames[eFormat - 1].toLowerCase().contains(
							"title")
							|| strCSVNames[eFormat - 1].toLowerCase().contains(
							"experience")) {
						// CSV formatting for long columns requires CRLF
						rlp.removeRule(RelativeLayout.ALIGN_PARENT_TOP);
						rlp.addRule(RelativeLayout.BELOW, currRow);
						// carriage return
						rlp.removeRule(RelativeLayout.RIGHT_OF);
						rlp.addRule(RelativeLayout.ALIGN_PARENT_LEFT);
						prevRow = currRow;
					} else {
						// special format that does not require CRLF
						// aligns to the right of the previous
						// TextView object
						rlp.addRule(RelativeLayout.RIGHT_OF, prevView);
						if ((intOrientation == Configuration.ORIENTATION_PORTRAIT)
								&& bLineWasSplit && (i == 1)) {
							rlp.addRule(RelativeLayout.BELOW, prevRow);
						}
					}
				} else { // (eFormat !> 0)
					// no special format, and short row aligns to the right of
					// the
					// previous TextView object
					rlp.addRule(RelativeLayout.RIGHT_OF, prevView);
					if ((intOrientation == Configuration.ORIENTATION_PORTRAIT)
							&& bLineWasSplit && (i == 1)) {
						rlp.addRule(RelativeLayout.BELOW, prevRow);
					}
				} // if (eFormat>0)

				// Append the content in a TextView object
				intUniqueId = i * 100 + j * 3;
				currView = addElement(ct, (ViewGroup) rlColumn, rlp, sText,
						intUniqueId, eFormat);

				currRow = currView;
				prevView = currView;
			} // for j==...do the next element in the input file record
			boolFirstRow = false;
			prevRow = currRow;

		} // for i==... do the next record from the input file

	}// formatFragment

	private ArrayList<List<String>> readAssets(int intFrag, String strFileName)
			throws IOException {
		Resources res = getResources();
		AssetManager am = res.getAssets();
		InputStream is = am.open(strFileName);
		ArrayList<List<String>> Content = null;
		Content = readInputCSVFile(is);
		return Content;
	}

	private String readAssets(String strFileName) throws IOException {
		Resources res = getResources();
		AssetManager am = res.getAssets();
		InputStream is = am.open(strFileName);
		String Content = "";
		Content = readInputFile(is);
		return Content;
	}

	private ArrayList<List<String>> readInputCSVFile(InputStream is) {
		// This reads a file that contains Comma-separated values
		// The first record in a CSV file contains the column names
		// and indicates the number of fields in each line
		ArrayList<List<String>> araCSVInput = new ArrayList<List<String>>();
		String strSeparator = "\\^";
		try {
			BufferedReader br = new BufferedReader(new InputStreamReader(is));
			String ContentLine; // buffered reader stops at newline character
			while ((ContentLine = br.readLine()) != null) {
				// separate the string into separate fields
				String[] items = ContentLine.split(strSeparator);
				List<String> singleLine = Arrays.asList(items);
				araCSVInput.add(singleLine);
			}
			br.close();
		} catch (IOException e) {
			e.printStackTrace();
		} finally {
			try {

				is.close();
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
		return araCSVInput;
	}

	private String readInputFile(InputStream is) throws IOException {
		StringBuilder ContentStream = new StringBuilder();

		try {
			BufferedReader br = new BufferedReader(new InputStreamReader(is));
			String ContentLine;

			while ((ContentLine = br.readLine()) != null) {
				ContentStream.append(ContentLine);
				ContentStream.append('\n');
			}
			br.close();
		} catch (IOException e) {
			e.printStackTrace();
		} // try

		String strFileContents = ContentStream.toString();
		return strFileContents;
	}

	private ArrayList<List<String>> readRawFile(int intFrag, String strResId)
			throws IOException {
		ArrayList<List<String>> Content = null;
		Resources res = getResources();
		int intFileResId = -1;
		try {
			intFileResId = res.getIdentifier(strResId, "raw", getActivity()
					.getPackageName());
		} catch (Exception e) {
			e.printStackTrace();
			intFileResId = -1;
		}
		if (intFileResId != -1) {
			InputStream is = res.openRawResource(intFileResId);
			Content = readInputCSVFile(is);
		}
		return Content;
	}

	private String readRawFile(String strResId) throws IOException {
		String Content = "";
		Resources res = getResources();
		int intFileResId = -1;
		try {
			intFileResId = res.getIdentifier(strResId, "raw", getActivity()
					.getPackageName());
		} catch (Exception e) {
			e.printStackTrace();
			intFileResId = -1;
		}
		if (intFileResId != -1) {
			InputStream is = res.openRawResource(intFileResId);
			Content = readInputFile(is);
		}
		return Content;
	}

	private ArrayList<List<String>> readSDCardFile(int intFrag,
			String strFileName) throws IOException {
		ArrayList<List<String>> Content = null;
		// Find the SD Card or other external directory using the API
		String fullPath = Environment.getExternalStorageDirectory()
				.getAbsolutePath();
		fullPath += "/" + strFileName;
		try {
			InputStream is = new FileInputStream(fullPath);
			Content = readInputCSVFile(is);
		} catch (Exception e) {
			e.printStackTrace();
		}
		return Content;
	}

	private String readSDCardFile(String strFileName) throws IOException {
		String Content = "";
		// Find the SD Card or other external Directory using the API
		String fullPath = Environment.getExternalStorageDirectory()
				.getAbsolutePath();
		fullPath += "/" + strFileName;
		try {
			InputStream is = new FileInputStream(fullPath);
			Content = readInputFile(is);
		} catch (Exception e) {
			e.printStackTrace();
		}
		return Content;
	}

	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);

		// load names of topic fragments into array
		Resources res = getResources();
		String[] fragNames = res.getStringArray(R.array.aryFragNames);

		if (getArguments().containsKey(ARG_ITEM_ID)) {
			// Retrieves the index of the text that is displayed
			// when the activity is initialized

			mItem = (getArguments().getString(ARG_ITEM_ID));

			// arrays are counted 0..<size>, but selections are counted
			// 1..#numberOfSelections
			// therefore the 3rd selection is associated with
			// array position 2
			int intFrag = Integer.parseInt(mItem) - 1;

			// create name of fragment to use
			strFragName = "fragment_" + fragNames[intFrag] + ".xml";
		}
	}

	@Override
	public View onCreateView(LayoutInflater inflater, ViewGroup container,
			Bundle savedInstanceState) {
		// default values
		String TopicContent = "incredible achievements!";
		String strFileName = "blank.txt";
		Context ctx = null;
		ArrayList<List<String>> TContent = null;
		View rootView = null;
		int intFragToInflate = 0;
		int intDenom = 0;
		boolean bMoveContent = false;
		Resources res = getResources();

		// load names of topic fragments into array
		String[] fragNames = res.getStringArray(R.array.aryFragNames);
		String[] topicNames = res.getStringArray(R.array.aryResumeTopics);

		// Show the content by switching to the new fragment
		mItem = (getArguments().getString(ARG_ITEM_ID));

		if (mItem != null) {
			// arrays are counted 0..<size>, but selections are counted
			// 1..#numberOfSelections
			// therefore the 3rd selection is associated with
			// array position 2
			int intFrag = Integer.parseInt(mItem) - 1;
			try {
				intFragToInflate = res.getIdentifier("fragment_"
						+ fragNames[intFrag], "layout", getActivity()
						.getPackageName());
			} catch (Exception e) {
				e.printStackTrace();
				intFragToInflate = -1;
			}
			if (intFragToInflate > 0) { // found layout
				// bio content is contained in fragment
				// bio is case 0 and is also the default case
				rootView = inflater.inflate(intFragToInflate, container, false);

				try {
					switch (intFrag) {
					// add content to view
					case 1:
						// qual content is contained in an SD Card file
						// content is NOT in a CSV file
						strFileName = fragNames[intFrag] + ".txt";
						TopicContent = readSDCardFile(strFileName);

						((TextView) rootView.findViewById(R.id.textContent))
								.setText(TopicContent);
						break;

					case 2:
						// educ content is contained in assets
						// content is NOT in a CSV file
						strFileName = fragNames[intFrag] + ".txt";
						TopicContent = readAssets(strFileName);

						((TextView) rootView.findViewById(R.id.textContent))
								.setText(TopicContent);
						break;

					case 3:
						// skil content is contained in an SD Card file
						// content is in a CSV file
						strFileName = fragNames[intFrag] + ".txt";
						TContent = readSDCardFile(intFragToInflate, strFileName);

						ctx = rootView.getContext();
						// format the content into screen content
						formatFragment(ctx, (ViewGroup) rootView, TContent);
						break;

					case 4:
						// expr content is contained in an SD Card file
						// content is in a CSV file
						strFileName = fragNames[intFrag] + ".txt";
						TContent = readSDCardFile(intFragToInflate, strFileName);

						ctx = rootView.getContext();

						// format the content into screen content
						formatFragment(ctx, (ViewGroup) rootView, TContent);
						break;

					case 5:
						// ntwk content is a raw resource file
						// content is in a CSV file
						strFileName = fragNames[intFrag];
						TContent = readRawFile(intFragToInflate, strFileName);

						ctx = rootView.getContext();
						// format the file content into screen content
						formatFragment(ctx, (ViewGroup) rootView, TContent);
						break;

					default:
						break;
					} // switch 1
				} catch (IOException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				} // try

			} // found layout

			int intId = res.getIdentifier("full_name", "string", this
					.getActivity().getPackageName());
			String strFullName = res.getString(intId);

			// access activity.topic.detail to change title
			this.getActivity().getActionBar()
					.setTitle(topicNames[intFrag] + " - " + strFullName);

		} // if (mItem != null)

		return rootView;

	} // onCreateView

	public void openBrowser(String strUrlAddress) {
		// private void openBrowser() {
		Uri uri = Uri.parse(strUrlAddress.toString());
		Intent intent = new Intent(Intent.ACTION_VIEW, uri);
		startActivity(intent);
	}

	public void openBrowser(View v) {
		TextView tv = (TextView) v;
		String strUrlAddress = tv.getText().toString();
		openBrowser(strUrlAddress);
	}

} // end class
