/**
 * Yvonne V. Richardson, BSCS, M. Ed.
 * yvonne.richardson@comcast.net
 * http://yvonnerichardson.weebly.com
 * Android Java Application
 * Resume for Phones and Tablets
 * Application Deployed 2014-09-25
 */
package com.example.resume;

import android.content.Intent;
import android.content.res.Resources;
import android.net.Uri;
import android.os.Bundle;
import android.support.v4.app.FragmentActivity;
import android.view.View;
import android.widget.TextView;
import android.widget.Toast;

/**
 * An activity representing a list of Topics. This activity has different
 * presentations for handset and tablet-size devices. On handsets, the activity
 * presents a list of items, which when touched, lead to a
 * {@link TopicDetailActivity} representing item details. On tablets, the
 * activity presents the list of items and item details side-by-side using two
 * vertical panes.
 * <p>
 * The activity makes heavy use of fragments. The list of items is a
 * {@link TopicListFragment} and the item details (if present) is a
 * {@link TopicDetailFragment}.
 * <p>
 * This activity also implements the required
 * {@link TopicListFragment.Callbacks} interface to listen for item selections.
 */
public class TopicListActivity extends FragmentActivity implements
		TopicListFragment.Callbacks {

	/**
	 * Whether or not the activity is in two-pane mode, i.e. running on a tablet
	 * device.
	 */
	private boolean mTwoPane;

	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.activity_topic_list);

		if (findViewById(R.id.topic_detail_container) != null) {
			// The detail container view will be present only in the
			// large-screen layouts (res/values-large and
			// res/values-sw600dp). If this view is present, then the
			// activity should be in two-pane mode.
			mTwoPane = true;

			// In two-pane mode, list items should be given the
			// 'activated' state when touched.

			((TopicListFragment) getSupportFragmentManager().findFragmentById(
					R.id.topic_list)).setActivateOnItemClick(true);
		}

		// TODO: If exposing deep links into your app, handle intents here.
	}

	/**
	 * Callback method from {@link TopicListFragment.Callbacks} indicating that
	 * the item with the given ID was selected.
	 */
	@Override
	public void onItemSelected(String id) {

		Resources res = getResources();
		int intArrayIdx = Integer.parseInt(id) - 1;
		int intId = res.getIdentifier("app_name", "string", getPackageName());
		String strAppName = res.getString(intId);

		intId = res.getIdentifier("full_name", "string", getPackageName());
		String strFullName = res.getString(intId);

		setTitle(strAppName + " - " + strFullName);

		if (mTwoPane) {
			// In two-pane mode, show the detail view in this activity by
			// adding or replacing the detail fragment using a
			// fragment transaction.
			Bundle arguments = new Bundle();
			arguments.putString(TopicDetailFragment.ARG_ITEM_ID, id);
			TopicDetailFragment fragment = new TopicDetailFragment();
			fragment.setArguments(arguments);
			getFragmentManager().beginTransaction()
					.replace(R.id.topic_detail_container, fragment).commit();
		} else {
			// In single-pane mode, simply start the detail activity
			// for the selected item ID.
			Intent detailIntent = new Intent(this, TopicDetailActivity.class);
			detailIntent.putExtra(TopicDetailFragment.ARG_ITEM_ID, id);
			startActivity(detailIntent);

		}
	} // onItemSelected

	public void openBrowser(String strUrlAddress) {
		// private void openBrowser() {
		Uri uri = Uri.parse(strUrlAddress.toString());
		Intent intent = new Intent(Intent.ACTION_VIEW, uri);
		startActivity(intent);
	}

	public void openBrowser(View v) {
		TextView tv = (TextView) v;
		String strUrlAddress = tv.getText().toString();
		openBrowser(strUrlAddress);
	}

	public void launchEmailApp(View v) {
		TextView tv = (TextView) v;
		String strEmAddr = tv.getText().toString();
		String strMail = "mailto";
		String strSubject = "Android Programming";
		String strBody = "Enter message here";
		String strCC = null;
		String strBCC = null;

		Intent em = new Intent(Intent.ACTION_SEND);
		em.putExtra(Intent.EXTRA_EMAIL, strEmAddr);
		em.putExtra(Intent.EXTRA_CC, strCC);
		em.putExtra(Intent.EXTRA_BCC, strBCC);
		em.putExtra(Intent.EXTRA_SUBJECT, strSubject);
		em.putExtra(Intent.EXTRA_TEXT, strBody);
		em.setData(Uri.parse(strMail + strEmAddr));
		em.setType("text/plain");
		try {
			startActivity(Intent.createChooser(em, "Send Email"));
		} catch (android.content.ActivityNotFoundException e) {
			Toast.makeText(TopicListActivity.this,
					"No email client is installed.", Toast.LENGTH_SHORT).show();
		}
	} // launchEmailApp

}
